""" This module tests all our quiz models"""
from django.test import TestCase
from .models import Sport, Criterion, Question


class SportModelTest(TestCase):
    """Tests the sport model"""

    def setUp(self):
        self.name = "HIIT"
        self.url = (
            "https://www.buchsys.de/fu-berlin/angebote/aktueller_zeitraum/_HIIT_"
            "-_High_Intensity_Interval_Training___HOME.html "
        )

        self.test_sport = Sport(
            name=self.name,
            url=self.url,
        )

        self.test_sport.save()

    def test_sport_can_be_created(self):
        """New sport is written to the database"""
        test_sport = Sport.objects.first()
        self.assertEqual(test_sport.name, self.name)
        self.assertEqual(test_sport.url, self.url)


class CriterionRatingTest(TestCase):
    """Tests the Relation between Sport and Criterion"""

    def setUp(self):
        self.name = "HIIT"
        self.url = (
            "https://www.buchsys.de/fu-berlin/angebote/aktueller_zeitraum/_HIIT_"
            "-_High_Intensity_Interval_Training___HOME.html "
        )

        self.test_sport = Sport(
            name=self.name,
            url=self.url,
        )

        self.test_sport.save()

        self.criterion = Criterion(name="Einzelsport")
        self.criterion.save()

    def test_can_rate_criterion_for_sport(self):
        """A rating for a specific criterion can be added to a sport"""
        self.test_sport.rate(self.criterion, 10)
        self.assertEqual(self.test_sport.criteria_ratings.first(), self.criterion)
        self.assertEqual(self.test_sport.get_rating(self.criterion), 10)

    def test_rating_can_be_changed(self):
        """
        If a sport is rated again then the corresponding relation is changed,
        instead of recreated
        """
        first_rating_object = self.test_sport.rate(self.criterion, 10)
        second_rating_object = self.test_sport.rate(self.criterion, 8)
        self.assertEqual(first_rating_object, second_rating_object)
        self.assertEqual(self.test_sport.get_rating(criterion=self.criterion), 8)


class CriterionAndQuestionModelTest(TestCase):
    """Tests the Criterion and the Question model which have a One to One Relation"""

    def setUp(self):
        self.name = "Einzelsport"
        self.criterion = Criterion(name=self.name)
        self.criterion.save()

    def test_criterion_can_be_saved_and_loaded(self):
        """New criterion can be loaded from the db"""
        test_criterion = Criterion.objects.first()
        self.assertEqual(test_criterion.name, self.name)

    def test_question_can_be_added(self):
        """
        If a question is added to a criterion, then it is accessible through the criterion.
        """
        text = "Ich trainiere gerne mit anderen im Team"
        question = Question(text=text, criterion=self.criterion)
        question.save()
        self.criterion.question = question
        self.assertEqual(question, self.criterion.question)

    def test_criterion_stays_if_question_deleted(self):
        """If assigned question is deleted the foreign key is set None"""
        text = "Ich trainiere gerne mit anderen im Team"
        question = Question(text=text, criterion=self.criterion)
        question.save()
        question.delete()
        self.criterion = Criterion.objects.first()

        with self.assertRaises(Criterion.question.RelatedObjectDoesNotExist):
            self.criterion.question

    def test_question_can_be_saved_and_loaded(self):
        """New Question is saved to the db and can be loaded"""
        text = "Ich trainiere gerne mit anderen im Team"
        Question(text=text, criterion=self.criterion).save()
        test_question = Question.objects.first()
        self.assertEqual(test_question.text, text)
